import { LOCALE_ID } from '@angular/core';
import {
  AfterViewInit,
  Component,
  Inject,
  OnInit,
  ViewChild,
} from '@angular/core';
import { MatTable } from '@angular/material/table';
import moment from 'moment';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import type { Speiseplan } from '../database-ha-interface';
import { DatabaseHaService } from '../database-ha.service';
import { SpeiseplanItem } from './speiseplan-interface';

/**
 * Displays a Speiseplan in a Material table.
 * Uses the DatabaseHaService for data retrieval.
 */
@Component({
  selector: 'app-speiseplan',
  templateUrl: './speiseplan.component.html',
  styleUrls: ['./speiseplan.component.css'],
  providers: [DatabaseHaService],
})
export class SpeiseplanComponent implements AfterViewInit, OnInit {
  @ViewChild(MatTable)
  table!: MatTable<SpeiseplanItem>;
  dataSource!: Observable<SpeiseplanItem[]>;

  /* NOTE
     We assume that the Speiseplan is for the current
     week because the endpoint doesn't return such information.
  */
  date: moment.Moment;

  /* Columns 0 to 4 are the weekdays Monday to Friday */
  displayedColumns = ['category', '0', '1', '2', '3', '4'];

  /* NOTE
     Dynamically generating these indices causes an
     ExpressionChangedAfterItHasBeenCheckedError exception
     in dev mode, so just define them once.
  */
  weekDays = [0, 1, 2, 3, 4];

  constructor(
    private service: DatabaseHaService,
    @Inject(LOCALE_ID) locale: string
  ) {
    this.date = moment().locale(locale);
  }

  ngOnInit(): void {
    this.dataSource = this.service
      .getSpeiseplan()
      .pipe(map((plan) => transformSpeiseplan(plan)));
  }

  ngAfterViewInit(): void {
    this.table.dataSource = this.dataSource;
  }

  /**
   * Get the weekday'th day since the beginning of the week
   * @param weekday The number of days since the beginning of the week
   * @returns Date of the weekday'th day since the beginning of the week
   */
  getWeekdayDate(weekday: number): Date {
    return this.date.startOf('isoWeek').add(weekday, 'days').toDate();
  }
}

/**
 * Resolve all ID's of plan and prepare
 * it for consumption.
 */
function transformSpeiseplan(plan: Speiseplan): SpeiseplanItem[] {
  return plan.Rows.map((row) => ({
    category: row.Name,
    days: toArray(
      row.Days.map((day) => [
        day.Weekday,
        day.ProductIds.map((pid) => {
          const product = plan.Products[pid.ProductId];

          return {
            name: product.Name,
            price: product.Price.Betrag,
            allergens: product.AllergenIds.map(
              (aid) => plan.Allergens[aid].Label
            ),
          };
        }),
      ])
    ),
  }));
}

/**
 * Convert an array of [index, value] pairs into an array
 * where value is at the provided index.
 * @param entries Array of pairs containing an index and a value
 */
function toArray<V>(entries: [number, V][]): V[] {
  const arr = [];
  for (const [k, v] of entries) {
    arr[k] = v;
  }

  return arr;
}
